import pandas as pd
from .model import Article, Comment


class PTTData:
    # initial PTTData object
    def __init__(self) -> None:
        self.__article_list: list[Article] = list()
        self.__comment_list: list[Comment] = list()

    # append data into list
    def append(self, obj: Article | Comment) -> None:
        """
        append data into PTTData, only accept Article and Comment object

        Parameters:
        obj (Article | Comment): data need to be append

        Returns:
        None
        """
        # check obj's instance
        if isinstance(obj, Article):
            self.__article_list.append(obj)
        elif isinstance(obj, Comment):
            self.__comment_list.append(obj)
        else:
            raise TypeError(f"Can't append {type(obj)}. Only accept <class 'Article'> or <class 'Comment'>.")

    # return article as dataframe
    def get_article_dataframe(self) -> pd.DataFrame:
        article_field = self.__article_list[0].article_field
        df_article = pd.DataFrame(data=self.get_article_list(), columns=article_field)
        return df_article

    # return comment as dataframe
    def get_comment_dataframe(self) -> pd.DataFrame:
        comment_field = self.__comment_list[0].comment_field
        df_comment = pd.DataFrame(data=self.get_comment_list(), columns=comment_field)
        return df_comment

    # return article as list
    def get_article_list(self) -> list:
        article_list = list()
        for idx, article in enumerate(self.__article_list):
            article_list.append(article.to_list())
        return article_list

    # return article as list
    def get_comment_list(self) -> list:
        comment_list = list()
        print(len(self.__comment_list))
        for idx, comment in enumerate(self.__comment_list):
            comment_list.append(comment.to_list())
        return comment_list

    # get origin list of Article
    def get_article(self) -> list[Article]:
        return self.__article_list

    # get origin list of Comment
    def get_comment(self) -> list[Comment]:
        return self.__comment_list

    # return article as dictionary (contain comment)
    def get_article_dict(self) -> list[dict]:
        article_dict = list()
        for article in self.__article_list:
            article_dict.append(article.to_dict())
        return article_dict

    # return comment as dictionary
    def get_comment_dict(self) -> list[dict]:
        comment_dict = list()
        for comment in self.__comment_list:
            comment_dict.append(comment.to_dict())
        return comment_dict

    # update self's data by another PTTData
    def update(self, ptt_data: "PTTData") -> None:
        for article in ptt_data.get_article():
            self.append(article)

        for comment in ptt_data.get_comment():
            self.append(comment)

    # return last element in article list
    def get_last_article(self) -> Article:
        if len(self.__article_list) == 0:
            return None
        return self.__article_list[-1]

    # return last element in article comment
    def get_last_comment(self) -> Comment:
        if len(self.__comment_list) == 0:
            return None
        return self.__comment_list[-1]


def main():
    ptt = PTTData()


if __name__ == "__main__":
    main()



import asyncio
import re
from datetime import datetime, timedelta

import aiohttp
from lxml import etree

from .model import Article, Comment
from .ptt_data import PTTData


class Crawler:
    PTT_URL: str = "https://www.ptt.cc"
    COOKIES: dict[str:str] = {"over18": "1"}
    article_filed: list[str] = [""]

    # initial Crawler
    def __init__(self, board: str, page_number: int) -> None:
        self.board = board
        self.page_number = page_number

    # get ptt board articles with specific page
    async def get_specific_page_data(self) -> PTTData:
        """
        Getting PTT board's articles with specific page.

        Parameters:
        None

        Returns:
        PTTData: preprocessed data from website response
        """
        print(f"Start crawling {self.board}: {self.page_number}")
        url = f"{Crawler.PTT_URL}/bbs/{self.board}/index{self.page_number}.html"
        try:
            result = await self.get_url_data(url)
        except Exception as e:
            print(e)
            print(f"{self.board}: getting {url} error.")
            return None
        processed_result = await self.processing_data(result)
        print(f"Finish crawling {self.board}: {self.page_number}")
        return processed_result

    # get original data from url
    async def get_url_data(self, url: str) -> str:
        """
        Getting data from url with async HTTP request.

        Parameters:
        url (str): url where data comes from

        Returns:
        str: original response text
        """
        # create async http session
        async with aiohttp.ClientSession() as session:
            # get data from async http request
            async with session.get(url=url, cookies=Crawler.COOKIES) as response:
                # waiting for the data to be received
                html = await response.text(encoding="utf-8")
                # return original data
                return html

    # processing data
    async def processing_data(self, original_text: str) -> PTTData:
        # part 1. remove on-top articles
        tree = self.__remove_on_top_article(etree.HTML(original_text))

        # part 2. get article links
        article_links, article_ids = self.__get_article_links(tree)
        # article_links, article_ids = ["https://www.ptt.cc/bbs/Gossiping/M.1663144920.A.A6E.html"], ["M.1663144920.A.A6E"]

        # part 3. get article content
        article_content = await self.__get_article_content(article_links)

        # part 4. filter article content
        ptt_data = self.__filter_article_content(article_content, article_ids)

        return ptt_data

    # part 1. remove on-top article in etree
    def __remove_on_top_article(self, tree: etree.HTML) -> etree.HTML:
        """
        Removing on-top article by detect on-top article separate line in article html div.

        Parameters:
        tree (etree.HTML)

        Returns:
        etree.HTML
        """
        on_top_article_sep = False
        article_separate_line_xpath = "r-list-sep"
        article_xpath = '//*[@id="main-container"]/div[2]/div'
        # loop all article node and remove on-top articles
        for node in tree.xpath(article_xpath):
            # get html div class name
            class_name = node.get("class")

            # "r-list-sep" means the separate line between normal articles and on-top articles
            if class_name == article_separate_line_xpath:
                on_top_article_sep = True

            # remove node if meet the separate line
            if on_top_article_sep:
                node.getparent().remove(node)

        # return processed etree
        return tree

    # part 2. get links from etree
    def __get_article_links(self, tree: etree.HTML) -> list[str]:
        """
        Get article links from etree by xpath

        Parameters:
        tree (etree.HTML)

        Returns:
        list[str]: list of PTT article link
        list[str]: list of PTT article id
        """
        link_xpath = '//*[@id="main-container"]/div[2]/div/div[2]/a'
        article_id_pattern = "/bbs/.*?/(.*?)\.html"
        article_links = list()
        article_ids = list()
        # loop and store all article link
        for node in tree.xpath(link_xpath):
            href = node.get("href")
            article_ids.append(re.search(article_id_pattern, href).group(1))
            article_links.append(f"{Crawler.PTT_URL}{href}")

        # return article links
        return article_links, article_ids

    # part 3. get article content
    async def __get_article_content(self, article_links: list[str]) -> list[str]:
        """
        Get article content from link

        Parameters:
        article_links (list[str]): list of PTT article link

        Returns:
        list[str]: list of PTT article content
        """
        # get event loop
        event_loop = asyncio.get_event_loop()

        # list all task
        tasks = [event_loop.create_task(self.get_url_data(link)) for link in article_links]

        content_list = list()

        # waiting all tasks to be done
        for task in tasks:
            result = await task
            content_list.append(result)

        # return article links
        return content_list

    # part 4. filter article content
    def __filter_article_content(self, article_content: list[str], article_ids: list[str]) -> PTTData:
        """
        Get author, title, post-time, content, comment in article

        Parameters:
        article_content (list[str]): list of PTT article content
        article_ids (list[str]): list of PTT article id

        Returns:
        PTTData
        """
        ptt_data = PTTData()
        main_content_xpath = '//*[@id="main-content"]'

        article_xpath_dict = {
            "author": '//*[@id="main-content"]/div[1]',
            "title": '//*[@id="main-content"]/div[3]',
            "post_time": '//*[@id="main-content"]/div[4]',
        }
        comment_xpath_dict = {
            "comment": "//div[@class='push']",
            "push_tag": "//div[@class='push']/span[1]",
            "push_user_id": "//div[@class='push']/span[2]",
            "push_content": "//div[@class='push']/span[3]",
            "push_ip_date_time": "//div[@class='push']/span[4]",
        }
        comment_ip_pattern = r"(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"
        comment_datetime_pattern = r"(\d{,2}/\d{,2} \d{,2}:\d{,2})"
        # loop content and extract useful information
        for article_id, content in zip(article_ids, article_content):
            try:
                lxml_tree = etree.HTML(content).xpath(main_content_xpath)[0]

                article_data = dict()
                # skip data if it is incomplete.
                try:
                    for key, value in article_xpath_dict.items():
                        article_data[key] = lxml_tree.xpath(value)[0].xpath("span")[1].text
                except IndexError as IE:
                    continue

                # get author
                full_name = article_data["author"][:-1].split(" (")
                user_id, user_name = full_name[0], full_name[0] if len(full_name) != 2 else full_name[1]
                # get title
                title = article_data["title"]
                # get post date
                try:
                    post_time = datetime.strptime(article_data["post_time"].replace(" ", "|").replace("||", "|0"), "%a|%b|%d|%H:%M:%S|%Y")
                except ValueError as VE:
                    previous_time = ptt_data.get_last_article()
                    # skip data if it is incomplete.
                    if previous_time is None:
                        continue
                    post_time = previous_time.post_time + timedelta(seconds=1)

                # get comment
                comment_list = []
                comment_data = [
                    range(len(lxml_tree.xpath(comment_xpath_dict["comment"]))),
                    lxml_tree.xpath(comment_xpath_dict["push_tag"]),
                    lxml_tree.xpath(comment_xpath_dict["push_user_id"]),
                    lxml_tree.xpath(comment_xpath_dict["push_content"]),
                    lxml_tree.xpath(comment_xpath_dict["push_ip_date_time"]),
                ]
                for idx, push_tag, push_user_id, push_content, push_ip_date_time in zip(*comment_data):
                    _push_tag = push_tag.text.replace(" ", "")
                    _push_user_id = push_user_id.text
                    _push_content = push_content.text
                    _push_ip_date_time = re.sub("[\n]", "", push_ip_date_time.text)
                    _push_ip = re.search(comment_ip_pattern, _push_ip_date_time)
                    _push_ip = _push_ip.group(1) if _push_ip else None
                    try:
                        _push_date_time = datetime.strptime(
                            str(post_time.year) + "/" + re.search(comment_datetime_pattern, _push_ip_date_time).group(1), "%Y/%m/%d %H:%M"
                        )
                    except:
                        _push_date_time = None

                    # append into ptt_data
                    comment = Comment(article_id, _push_tag, _push_user_id, idx + 1, _push_content, _push_date_time, _push_ip)
                    ptt_data.append(comment)
                    comment_list.append(comment)

                # get context
                # remove all comments, leave only article context
                delete_flag = False
                for i in lxml_tree.xpath("./*"):
                    if i.get("class") == "f2":
                        delete_flag = True
                    if delete_flag:
                        i.getparent().remove(i)
                context_xpath = "//div[@class='article-metaline'][3]/following-sibling::text()"
                context_list = lxml_tree.xpath(context_xpath)
                # remove all \n, \t
                context = "".join(map(lambda x: re.sub(r"[\s\t]", "", x), context_list))
                # get ip
                ip_address = re.search(comment_ip_pattern, content)
                ip_address = ip_address.group(1) if ip_address else ""

                # append into ptt_data
                article = Article(article_id, title, user_id, user_name, self.board, post_time, context, ip_address, comment_list)
                ptt_data.append(article)
            except Exception as e:
                print("Getting article error: ", e)

        return ptt_data


def main():
    pass


if __name__ == "__main__":
    main()



import asyncio
import re

import requests
from .crawler import Crawler
from .ptt_data import PTTData


class AioPTTCrawler:
    # class variable
    PTT_URL: str = "https://www.ptt.cc"
    COOKIES: dict[str:str] = {"over18": "1"}

    # initial PTTCrawler object
    def __init__(self) -> None:
        pass

    # get newest pages from ptt board
    def get_board_latest_articles(self, board: str, page_count: int = 10) -> PTTData:
        """
        Getting PTT board's articles with amount of pages.

        Parameters:
        board (str): PTT board's name
        page_count (int): amount of pages.

        Returns:
        PTTData: custom class to store data from PTT
        """
        # get latest page number
        end_index = self.get_latest_index(board)
        # calculate start page number
        start_index = end_index - page_count + 1

        # return PTTData
        return self.get_board_articles(board, start_index, end_index)

    # get articles by range of index
    def get_board_articles(self, board: str, start_index: int, end_index: int) -> PTTData:
        """
        Getting PTT board's articles with amount of pages.

        Parameters:
        board (str): PTT board's name
        start_index (int): start index.
        end_index (int): end index.

        Returns:
        PTTData: custom class to store data from PTT
        """
        # create async event loop
        self.event_loop = asyncio.get_event_loop()
        # list all crawler
        crawlers = [Crawler(board, i) for i in range(start_index, end_index + 1)]
        # list all tasks
        tasks = [crawler.get_specific_page_data() for crawler in crawlers]
        # run tasks and get the results
        results: list[PTTData] = self.event_loop.run_until_complete(asyncio.gather(*tasks))

        # release memory
        del crawlers, tasks

        ptt_data = PTTData()
        for sub_ptt_data in results:
            if sub_ptt_data:
                ptt_data.update(sub_ptt_data)
        return ptt_data

    # get latest page index from ptt board
    def get_latest_index(self, board: str) -> int:
        # board's index.html always point to the newest page.
        content = requests.get(
            url=f"{AioPTTCrawler.PTT_URL}/bbs/{board}/index.html",
            cookies=AioPTTCrawler.COOKIES,
        ).content.decode("utf-8")

        # search for the previous page number.
        previous_page = re.search(f'href="/bbs/{board}/index(\d+).html">&lsaquo;', content)

        # check if there is latest page in this board.
        if previous_page is None:
            raise ValueError(f"Can't get board:<{board}>'s latest page index.")

        # return latest page number
        return int(previous_page.group(1)) + 1


def main():
    BOARD = "Gossiping"
    ptt_crawler = AioPTTCrawler()
    ptt_data = ptt_crawler.get_board_latest_articles(board=BOARD, page_count=100)
    d = ptt_data.get_article_dict()
    print(d)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import os
import re
import sys
import json
import requests
import argparse
import time
import codecs
from bs4 import BeautifulSoup
from six import u

__version__ = '1.0'

# if python 2, disable verify flag in requests.get()
VERIFY = True
if sys.version_info[0] < 3:
    VERIFY = False
    requests.packages.urllib3.disable_warnings()


class PttWebCrawler(object):

    PTT_URL = 'https://www.ptt.cc'

    """docstring for PttWebCrawler"""
    def __init__(self, cmdline=None, as_lib=False):
        parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter, description='''
            A crawler for the web version of PTT, the largest online community in Taiwan.
            Input: board name and page indices (or articla ID)
            Output: BOARD_NAME-START_INDEX-END_INDEX.json (or BOARD_NAME-ID.json)
        ''')
        parser.add_argument('-b', metavar='BOARD_NAME', help='Board name', required=True)
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('-i', metavar=('START_INDEX', 'END_INDEX'), type=int, nargs=2, help="Start and end index")
        group.add_argument('-a', metavar='ARTICLE_ID', help="Article ID")
        parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)

        if not as_lib:
            if cmdline:
                args = parser.parse_args(cmdline)
            else:
                args = parser.parse_args()
            board = args.b
            if args.i:
                start = args.i[0]
                if args.i[1] == -1:
                    end = self.getLastPage(board)
                else:
                    end = args.i[1]
                self.parse_articles(start, end, board)
            else:  # args.a
                article_id = args.a
                self.parse_article(article_id, board)

    def parse_articles(self, start, end, board, path='.', timeout=3):
            filename = board + '-' + str(start) + '-' + str(end) + '.json'
            filename = os.path.join(path, filename)
            # self.store(filename, u'{"articles": [', 'w')
            for i in range(end-start+1):
                index = start + i
                print('Processing index:', str(index))
                resp = requests.get(
                    url = self.PTT_URL + '/bbs/' + board + '/index' + str(index) + '.html',
                    cookies={'over18': '1'}, verify=VERIFY, timeout=timeout
                )
                if resp.status_code != 200:
                    print('invalid url:', resp.url)
                    continue
                soup = BeautifulSoup(resp.text, 'html.parser')
                divs = soup.find_all("div", "r-ent")
                for div in divs:
                    try:
                        # ex. link would be <a href="/bbs/PublicServan/M.1127742013.A.240.html">Re: [問題] 職等</a>
                        href = div.find('a')['href']
                        link = self.PTT_URL + href
                        article_id = re.sub('\.html', '', href.split('/')[-1])
                        self.parse(link, article_id, board)
                        # if div == divs[-1] and i == end-start:  # last div of last page
                        #     self.store(filename, self.parse(link, article_id, board), 'a')
                        # else:
                        #     self.store(filename, self.parse(link, article_id, board) + ',\n', 'a')
                    except:
                        pass
                # time.sleep(0.1)
            # self.store(filename, u']}', 'a')
            return filename

    def parse_article(self, article_id, board, path='.'):
        link = self.PTT_URL + '/bbs/' + board + '/' + article_id + '.html'
        filename = board + '-' + article_id + '.json'
        filename = os.path.join(path, filename)
        # self.store(filename, self.parse(link, article_id, board), 'w')
        return filename

    @staticmethod
    def parse(link, article_id, board, timeout=3):
        # print('Processing article:', article_id)
        resp = requests.get(url=link, cookies={'over18': '1'}, verify=VERIFY, timeout=timeout)
        if resp.status_code != 200:
            print('invalid url:', resp.url)
            # return json.dumps({"error": "invalid url"}, sort_keys=True, ensure_ascii=False)
        soup = BeautifulSoup(resp.text, 'html.parser')
        main_content = soup.find(id="main-content")
        metas = main_content.select('div.article-metaline')
        author = ''
        title = ''
        date = ''
        if metas:
            author = metas[0].select('span.article-meta-value')[0].string if metas[0].select('span.article-meta-value')[0] else author
            title = metas[1].select('span.article-meta-value')[0].string if metas[1].select('span.article-meta-value')[0] else title
            date = metas[2].select('span.article-meta-value')[0].string if metas[2].select('span.article-meta-value')[0] else date

            # remove meta nodes
            for meta in metas:
                meta.extract()
            for meta in main_content.select('div.article-metaline-right'):
                meta.extract()

        # remove and keep push nodes
        pushes = main_content.find_all('div', class_='push')
        for push in pushes:
            push.extract()

        try:
            ip = main_content.find(text=re.compile(u'※ 發信站:'))
            ip = re.search('[0-9]*\.[0-9]*\.[0-9]*\.[0-9]*', ip).group()
        except:
            ip = "None"

        # 移除 '※ 發信站:' (starts with u'\u203b'), '◆ From:' (starts with u'\u25c6'), 空行及多餘空白
        # 保留英數字, 中文及中文標點, 網址, 部分特殊符號
        filtered = [ v for v in main_content.stripped_strings if v[0] not in [u'※', u'◆'] and v[:2] not in [u'--'] ]
        expr = re.compile(u(r'[^\u4e00-\u9fa5\u3002\uff1b\uff0c\uff1a\u201c\u201d\uff08\uff09\u3001\uff1f\u300a\u300b\s\w:/-_.?~%()]'))
        for i in range(len(filtered)):
            filtered[i] = re.sub(expr, '', filtered[i])

        filtered = [_f for _f in filtered if _f]  # remove empty strings
        filtered = [x for x in filtered if article_id not in x]  # remove last line containing the url of the article
        content = ' '.join(filtered)
        content = re.sub(r'(\s)+', ' ', content)
        # print 'content', content

        # push messages
        p, b, n = 0, 0, 0
        messages = []
        for push in pushes:
            if not push.find('span', 'push-tag'):
                continue
            push_tag = push.find('span', 'push-tag').string.strip(' \t\n\r')
            push_userid = push.find('span', 'push-userid').string.strip(' \t\n\r')
            # if find is None: find().strings -> list -> ' '.join; else the current way
            push_content = push.find('span', 'push-content').strings
            push_content = ' '.join(push_content)[1:].strip(' \t\n\r')  # remove ':'
            push_ipdatetime = push.find('span', 'push-ipdatetime').string.strip(' \t\n\r')
            messages.append( {'push_tag': push_tag, 'push_userid': push_userid, 'push_content': push_content, 'push_ipdatetime': push_ipdatetime} )
            if push_tag == u'推':
                p += 1
            elif push_tag == u'噓':
                b += 1
            else:
                n += 1

        # count: 推噓文相抵後的數量; all: 推文總數
        message_count = {'all': p+b+n, 'count': p-b, 'push': p, 'boo': b, "neutral": n}

        # print 'msgs', messages
        # print 'mscounts', message_count

        # json data
        data = {
            'url': link,
            'board': board,
            'article_id': article_id,
            'article_title': title,
            'author': author,
            'date': date,
            'content': content,
            'ip': ip,
            'message_count': message_count,
            'messages': messages
        }
        # print 'original:', d
        # return json.dumps(data, sort_keys=True, ensure_ascii=False)

    @staticmethod
    def getLastPage(board, timeout=3):
        content = requests.get(
            url= 'https://www.ptt.cc/bbs/' + board + '/index.html',
            cookies={'over18': '1'}, timeout=timeout
        ).content.decode('utf-8')
        first_page = re.search(r'href="/bbs/' + board + '/index(\d+).html">&lsaquo;', content)
        if first_page is None:
            return 1
        return int(first_page.group(1)) + 1

    @staticmethod
    def store(filename, data, mode):
        with codecs.open(filename, mode, encoding='utf-8') as f:
            f.write(data)

    @staticmethod
    def get(filename, mode='r'):
        with codecs.open(filename, mode, encoding='utf-8') as f:
            return json.load(f)

if __name__ == '__main__':
    c = PttWebCrawler()



from dataclasses import dataclass, field
from datetime import datetime


@dataclass
class Comment:
    article_id: str
    tag: str
    user_id: str
    comment_order: int
    context: str
    post_time: datetime
    ip_address: str
    comment_field: list[str] = field(default_factory=list, init=False)
    data_list: list = field(default_factory=list, init=False)

    def __post_init__(self) -> None:
        self.comment_field = [
            "article_id",
            "tag",
            "user_id",
            "comment_order",
            "context",
            "datetime",
            "ip_address",
        ]
        self.data_list = [self.article_id, self.tag, self.user_id, self.comment_order, self.context, self.post_time, self.ip_address]

    # return all value as dict
    def to_dict(self) -> dict:
        return_dict = dict()
        for key, value in zip(self.comment_field, self.data_list):
            return_dict[key] = value
        return return_dict

    # return all value as list
    def to_list(self) -> list:
        return


def main():
    pass


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

with open("README.md", "r", encoding="UTF-8") as file:
    long_description = file.read()

name = "AioPTTCrawler"
setup(
    name=name,
    version="0.0.9",
    author="DOUIF",
    author_email="a15975345678@gmail.com",
    description="PTT crawler using asyncio",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DOUIF/aio-ptt-crawler",
    packages=find_packages(),
    install_requires=[
        "aiohttp>=3.8.3",
        "lxml>=4.9.1",
        "requests>=2.28.1",
        "pandas>=1.5.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10",
)



from dataclasses import dataclass, field
from datetime import datetime
from .comment import Comment


@dataclass
class Article:
    article_id: str
    article_title: str
    user_id: str
    user_name: str
    board: str
    post_time: datetime
    context: str
    ip_address: str
    comment_list: list[Comment] = field(default_factory=list)
    article_field: list[str] = field(default_factory=list, init=False)
    data_list: list = field(default_factory=list, init=False)

    def __post_init__(self) -> None:
        self.article_field = [
            "article_id",
            "article_title",
            "user_id",
            "user_name",
            "board",
            "datetime",
            "context",
            "ip_address",
        ]
        self.data_list = [
            self.article_id,
            self.article_title,
            self.user_id,
            self.user_name,
            self.board,
            self.post_time,
            self.context,
            self.ip_address,
        ]

    # return all value as list(exclude comment_list)
    def to_list(self) -> list:
        return self.data_list

    # return all value as dict(include comment_list)
    def to_dict(self) -> dict:
        result = dict()
        for key, value in zip(self.article_field, self.data_list):
            result[key] = value
        result["comment_list"] = list()

        # add comment
        for comment in self.comment_list:
            result["comment_list"].append(comment.to_dict())

        # return data
        return result

    def __str__(self):
        return f"{self.article_id:=}, { self.article_title:=}, { self.user_id:=}, { self.user_name:=}, { self.board:=}, { self.post_time:=}, { self.context:=}, { self.ip_address:=}"


def main():
    pass


if __name__ == "__main__":
    main()
